
library(sp)
library(raster)
library(rgbif)
library(maptools)
library(rgeos)
library(dismo)
library(rgdal)

# open data file

data_c_laminata_new2 <-read.csv('sdm.data_c_laminata_new2.csv')
data_c_laminata_new2 <- data_c_laminata_new2[,-1] 
data_c_laminata_new2$species <- 1 
coordinates(data_c_laminata_new2) <- ~lon+lat 
new_climate <- raster::getData('worldclim', var='bio', res=2.5)
new_climate_cut = crop(new_climate, extent(60,66,20,40)) #60E, 66N, 20W, 40S 
climate_3 <- subset(new_climate_cut, c('bio1', 'bio2', 'bio4', 'bio8', 'bio9', 'bio15', 'bio18', 'bio19')) 
#MODELLING
library(sdm)
installAll()
d_C_lam_new12<-sdmData(species~., data_c_laminata_new2, predictors=climate_3, bg = list (n=400)) 
m_c_laminata_clim<-sdm(species~., d_C_lam_new12, methods = c('gam', 'mlp', 'svm'),
                        replication='sub', test.percent=25,n=5)
p_c_laminata_clim1 <-predict(m_c_laminata_clim, climate_3)
en_c_laminata_clim1 <- ensemble(m_c_laminata_clim,  climate_3,
                                setting = list(method= 'weight', stat= 'AUC') )
roc(m_c_laminata_clim) # AUC
## 2081-2100 years####
# HadGEM3-GC31-LL model, res 2.5, ssp585, 2081-2100 years
HadGEM3_100<- stack('wc2.1_2.5m_bioc_HadGEM3-GC31-LL_ssp585_2081-2100.tif')
HadGEM3_100_cut = crop(HadGEM3_100, extent(60,66,20,40)) 
pred_HadGEM3_100_cut <- subset(HadGEM3_100_cut, c('wc2_1', 'wc2_2', 'wc2_4', 'wc2_8', 'wc2_9', 'wc2_15', 'wc2_18', 'wc2_19')) 
names(pred_HadGEM3_100_cut)<-names(climate_3)
p_c_laminata_Fclim2 <- predict(m_c_laminata_clim, pred_HadGEM3_100_cut_reserv)
en_c_laminata_Fclim2 <- ensemble(m_c_laminata_clim,  pred_HadGEM3_100_cut,
                                 setting = list(method= 'weight', stat= 'AUC') )
HadGEM3_100_45<- stack('wc2.1_2.5m_bioc_HadGEM3-GC31-LL_ssp245_2081-2100.tif')
HadGEM3_100_45_cut = crop(HadGEM3_100_45, extent(60,66,20,40)) 
pred_HadGEM3_100_45_cut <- subset(HadGEM3_100_45_cut, c('wc2_1', 'wc2_2', 'wc2_4', 'wc2_8', 'wc2_9', 'wc2_15', 'wc2_18', 'wc2_19'))
names(pred_HadGEM3_100_45_cut)<-names(climate_3)
p_c_laminata_Fclim3 <- predict(m_c_laminata_clim, pred_HadGEM3_100_45_cut)
en_c_laminata_Fclim3 <- ensemble(m_c_laminata_clim,  pred_HadGEM3_100_45_cut,
                                 setting = list(method= 'weight', stat= 'AUC') )
ACCESS_100_85<- stack('wc2.1_2.5m_bioc_ACCESS-ESM1-5_ssp585_2081-2100.tif')
ACCESS_100_45<- stack('wc2.1_2.5m_bioc_ACCESS-ESM1-5_ssp245_2081-2100.tif')
ACCESS_100_85_cut = crop(ACCESS_100_85, extent(60,66,20,40))
pred_ACCESS_100_85_cut <- subset(ACCESS_100_85_cut, c('wc2_1', 'wc2_2', 'wc2_4', 'wc2_8', 'wc2_9', 'wc2_15', 'wc2_18', 'wc2_19'))
names(pred_ACCESS_100_85_cut)<-names(climate_3)
ACCESS_100_45_cut = crop(ACCESS_100_45, extent(60,66,20,40))
pred_ACCESS_100_45_cut <- subset(ACCESS_100_45_cut, c('wc2_1', 'wc2_2', 'wc2_4', 'wc2_8', 'wc2_9', 'wc2_15', 'wc2_18', 'wc2_19'))
names(pred_ACCESS_100_45_cut)<-names(climate_3)
p_c_laminata_Fclim4 <- predict(m_c_laminata_clim,pred_ACCESS_100_85_cut)
en_c_laminata_Fclim4 <- ensemble(m_c_laminata_clim,  pred_ACCESS_100_85_cut,
                                 setting = list(method= 'weight', stat= 'AUC'))
p_c_laminata_Fclim5 <- predict(m_c_laminata_clim,pred_ACCESS_100_45_cut)
en_c_laminata_Fclim5 <- ensemble(m_c_laminata_clim,  pred_ACCESS_100_45_cut,
                                 setting = list(method= 'weight', stat= 'AUC') )

## 2041-2060 years####
HadGEM3_60<- stack('wc2.1_2.5m_bioc_HadGEM3-GC31-LL_ssp585_2041-2060.tif')
HadGEM3_60_cut = crop(HadGEM3_60, extent(60,66,20,40))
pred_HadGEM3_60_cut <- subset(HadGEM3_60_cut, c('wc2_1', 'wc2_2', 'wc2_4', 'wc2_8', 'wc2_9', 'wc2_15', 'wc2_18', 'wc2_19'))
names(pred_HadGEM3_60_cut)<-names(climate_3)
p_c_laminata_clim_60H <- predict(m_c_laminata_clim, pred_HadGEM3_60_cut)
en_c_laminata_clim_60H <- ensemble(m_c_laminata_clim,  pred_HadGEM3_60_cut,
                                 setting = list(method= 'weight', stat= 'AUC') )
HadGEM3_60_245<- stack('wc2.1_2.5m_bioc_HadGEM3-GC31-LL_ssp245_2041-2060.tif')
HadGEM3_60_245_cut = crop(HadGEM3_60_245, extent(60,66,20,40))
pred_HadGEM3_60_245_cut <- subset(HadGEM3_60_245_cut, c('wc2_1', 'wc2_2', 'wc2_4', 'wc2_8', 'wc2_9', 'wc2_15', 'wc2_18', 'wc2_19'))
names(pred_HadGEM3_60_245_cut)<-names(climate_3)
p_c_laminata_clim_60H_245 <- predict(m_c_laminata_clim, pred_HadGEM3_60_245_cut)
en_c_laminata_clim_60H_245 <- ensemble(m_c_laminata_clim,  pred_HadGEM3_60_245_cut,
                                   setting = list(method= 'weight', stat= 'AUC') )
ACCESS_60_85<- stack('wc2.1_2.5m_bioc_ACCESS-ESM1-5_ssp585_2041-2060.tif')
ACCESS_60_45<- stack('wc2.1_2.5m_bioc_ACCESS-ESM1-5_ssp245_2041-2060.tif')
ACCESS_60_85_cut = crop(ACCESS_60_85, extent(60,66,20,40))
pred_ACCESS_60_85_cut <- subset(ACCESS_60_85_cut, c('wc2_1', 'wc2_2', 'wc2_4', 'wc2_8', 'wc2_9', 'wc2_15', 'wc2_18', 'wc2_19'))
names(pred_ACCESS_60_85_cut)<-names(climate_3)
ACCESS_60_45_cut = crop(ACCESS_60_45, extent(60,66,20,40))
pred_ACCESS_60_45_cut <- subset(ACCESS_60_45_cut, c('wc2_1', 'wc2_2', 'wc2_4', 'wc2_8', 'wc2_9', 'wc2_15', 'wc2_18', 'wc2_19'))
names(pred_ACCESS_60_45_cut)<-names(climate_3)
p_c_laminata_clim_A60_585 <- predict(m_c_laminata_clim,pred_ACCESS_60_85_cut)
en_c_laminata_A60_585 <- ensemble(m_c_laminata_clim,  pred_ACCESS_60_85_cut,
                                 setting = list(method= 'weight', stat= 'AUC'))
p_c_laminata_A60_245 <- predict(m_c_laminata_clim,pred_ACCESS_60_45_cut)
en_c_laminata_A60_245 <- ensemble(m_c_laminata_clim,  pred_ACCESS_60_45_cut,
                                 setting = list(method= 'weight', stat= 'AUC') )
ev_clim1 <- getEvaluation(m_c_laminata_clim, stat= c ('AUC', 'TSS', 'threshold'), opt=2)
mean(ev_clim1$threshold)#0.3486417
pa_en_clim1 <- raster(en_c_laminata_clim1)
pa_en_clim1[] <- ifelse(en_c_laminata_clim1[] >= 0.3486417, 1, 0)
pa_en_Fclim2 <- raster(en_c_laminata_Fclim2)
pa_en_Fclim2[] <- ifelse(en_c_laminata_Fclim2[] >= 0.3486417, 1, 0)
pa_en_Fclim3 <- raster(en_c_laminata_Fclim3)
pa_en_Fclim3[] <- ifelse(en_c_laminata_Fclim3[] >= 0.3486417, 1, 0)
pa_en_Fclim4 <- raster(en_c_laminata_Fclim4)
pa_en_Fclim4[] <- ifelse(en_c_laminata_Fclim4[] >= 0.3486417, 1, 0)
pa_en_Fclim5 <- raster(en_c_laminata_Fclim5)
pa_en_Fclim5[] <- ifelse(en_c_laminata_Fclim5[] >= 0.3486417, 1, 0)
pa_en_clim_60H_585 <- raster(en_c_laminata_clim_60H)
pa_en_clim_60H_585[] <- ifelse(en_c_laminata_clim_60H[] >= 0.3486417, 1, 0)
pa_en_clim_60H_245 <- raster(en_c_laminata_clim_60H_245 )
pa_en_clim_60H_245[] <- ifelse(en_c_laminata_clim_60H_245 [] >= 0.3486417, 1, 0)
pa_en_clim_A60_585 <- raster(en_c_laminata_A60_585)
pa_en_clim_A60_585[] <- ifelse(en_c_laminata_A60_585 [] >= 0.3486417, 1, 0)
pa_en_clim_A60_245 <- raster(en_c_laminata_A60_245)
pa_en_clim_A60_245[] <- ifelse(en_c_laminata_A60_245 [] >= 0.3486417, 1, 0)
# Relative Variable Importance
vi_clim <- getVarImp(m_c_laminata_clim,id=1:15,wtest='test.dep') #ВКЛАД ПРЕДИКТОРОВ В МОДЕЛИ УСРЕДНЕННО
plot(vi_clim,'cor')
vi_clim_gam <- getVarImp(m_c_laminata_clim ,id=1:5,wtest='test.dep') # вклад предикторов в GAM
plot(vi_clim_gam,'cor')
vi_clim_mlp <- getVarImp(m_c_laminata_clim ,id=6:10,wtest='test.dep') # вклад предикторов в MLP
plot(vi_clim_mlp,'cor')
vi_clim_svm <- getVarImp(m_c_laminata_clim ,id=11:15,wtest='test.dep') # вклад предикторов в SVM
plot(vi_clim_svm,'cor')

library(spatstat)
writeRaster(x=raster(im.apply(X=list(as.im(en_c_laminata_clim_60H_245),
                              as.im(en_c_laminata_A60_245),
                              as.im(en_c_laminata_clim_60Can245),
                              as.im(en_c_laminata_clim_60BCC245)),FUN=sd)),
            filename='SD.tif',
            format='GTiff')
writeRaster(x=raster(im.apply(X=list(as.im(en_c_laminata_clim_60H_245),
                                     as.im(en_c_laminata_A60_245),
                                     as.im(en_c_laminata_clim_60Can245),
                                     as.im(en_c_laminata_clim_60BCC245)),FUN=mean)),
            filename='Mean.tif',
            format='GTiff')
writeRaster(x=raster(im.apply(X=list(as.im(en_c_laminata_clim_60H),
                                     as.im(en_c_laminata_A60_585),
                                     as.im(en_c_laminata_60Can585),
                                     as.im(en_c_laminata_clim_60BCC585)),FUN=sd)),
            filename='2060_585_SD.tif',
            format='GTiff')
writeRaster(x=raster(im.apply(X=list(as.im(en_c_laminata_clim_60H),
                                     as.im(en_c_laminata_A60_585),
                                     as.im(en_c_laminata_clim_60Can585),
                                     as.im(en_c_laminata_clim_60BCC585)),FUN=mean)),
            filename='2060_585_Mean.tif',
            format='GTiff')
writeRaster(x=raster(im.apply(X=list(as.im(en_c_laminata_Fclim3),
                                     as.im(en_c_laminata_Fclim5),
                                     as.im(en_c_laminata_clim_100Can245),
                                     as.im(en_c_laminata_clim_100BCC245)),FUN=sd)),
            filename='2100_245_SD.tif',
            format='GTiff')
writeRaster(x=raster(im.apply(X=list(as.im(en_c_laminata_Fclim3),
                                     as.im(en_c_laminata_Fclim5),
                                     as.im(en_c_laminata_clim_100Can245),
                                     as.im(en_c_laminata_clim_100BCC245)),FUN=mean)),
            filename='2100_245_Mean.tif',
            format='GTiff')
writeRaster(x=raster(im.apply(X=list(as.im(en_c_laminata_Fclim2),
                                     as.im(en_c_laminata_Fclim4),
                                     as.im(en_c_laminata_100Can585),
                                     as.im(en_c_laminata_clim_100BCC585)),FUN=sd)),
            filename='2100_585_SD.tif',
            format='GTiff')
writeRaster(x=raster(im.apply(X=list(as.im(en_c_laminata_Fclim2),
                                     as.im(en_c_laminata_Fclim4),
                                     as.im(en_c_laminata_100Can585),
                                     as.im(en_c_laminata_clim_100BCC585)),FUN=mean)),
            filename='2100_585_Mean.tif',
            format='GTiff')

